import { ContentData } from "./ContentData";
/**
 * A class for managing the set of known content data types.
 *
 * It offers a set of constants that characterize the content
 * data type (i.e. they are predicates that match when the
 * content data has the respective type).
 *
 * These are used for registering the corresponding validators
 * in the `ContentDataValidators` class, and for determining
 * a name (string) that describes the content type.
 *
 * @internal
 */
export declare class ContentDataTypes {
    static readonly CONTENT_TYPE_GLB: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_B3DM: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_I3DM: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_CMPT: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_PNTS: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_GEOM: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_VCTR: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_GEOJSON: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_3TZ: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_GLTF: (contentData: ContentData) => Promise<boolean>;
    static readonly CONTENT_TYPE_TILESET: (contentData: ContentData) => Promise<boolean>;
    /**
     * Returns a name for the given content data.
     *
     * This is a string that corresponds to one of the constants in
     * this class, namely the name of the constant that matches the
     * given ContentData.
     *
     * If no matching name can be determined, then `undefined` is
     * returned.
     *
     * @param contentData - The content data
     * @returns The name
     */
    static nameFor(contentData: ContentData): Promise<string | undefined>;
    /**
     * Creates a predicate that checks whether the magic header of
     * a ContentData matches the given magic header string.
     *
     * @param magic - The magic header string
     * @returns The predicate
     */
    private static byMagic;
    /**
     * Creates a predicate that checks whether the extension of
     * a ContentData matches the given extension (which should
     * include the '.' dot).
     *
     * @param extension - The extension
     * @returns The predicate
     */
    private static byExtension;
    /**
     * Creates a predicate that says whether a ContentData is
     * (probably) a tileset.
     *
     * @returns The predicate
     */
    private static byBeingTileset;
    /**
     * Creates a predicate that says whether a ContentData is
     * (probably) a glTF JSON.
     *
     * @returns The predicate
     */
    private static byBeingGltf;
    /**
     * Returns whether the given content data is probably a tileset.
     *
     * The exact conditions for this method returning `true` are
     * intentionally not specified.
     *
     * @param contentData - The content data
     * @returns Whether the content data is probably a tileset
     */
    static isProbablyTileset(contentData: ContentData): Promise<boolean>;
    /**
     * Returns whether the given content data is probably a glTF
     * (not a GLB, but a glTF JSON).
     *
     * The exact conditions for this method returning `true` are
     * intentionally not specified.
     *
     * @param contentData - The content data
     * @returns Whether the content data is probably glTF
     */
    static isProbablyGltf(contentData: ContentData): Promise<boolean>;
}
//# sourceMappingURL=ContentDataTypes.d.ts.map