"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContentDataValidators = void 0;
const defined_1 = require("../base/defined");
const Validators_1 = require("./Validators");
const ContentDataTypes_1 = require("./ContentDataTypes");
const TilesetArchiveValidator_1 = require("./TilesetArchiveValidator");
const B3dmValidator_1 = require("../tileFormats/B3dmValidator");
const I3dmValidator_1 = require("../tileFormats/I3dmValidator");
const PntsValidator_1 = require("../tileFormats/PntsValidator");
const CmptValidator_1 = require("../tileFormats/CmptValidator");
const GltfValidator_1 = require("../tileFormats/GltfValidator");
const IoValidationIssue_1 = require("../issues/IoValidationIssue");
/**
 * A class for managing `Validator` instances that are used for
 * validating the data that is pointed to by a `content.uri`.
 *
 * The only public methods (for now) are `registerDefaults`,
 * which registers all known content data validators, and
 * `findContentDataValidator`, which returns the validator
 * that should be used for a given `ContentData` object.
 *
 * @internal
 */
class ContentDataValidators {
    /**
     * Registers all default content data validators
     */
    static registerDefaults() {
        if (ContentDataValidators._registeredDefaults) {
            return;
        }
        // The validators will be checked in the order in which they are
        // registered. In the future, there might be a mechanism for
        // 'overriding' a previously registered validator.
        ContentDataValidators.registerForBuffer(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_GLB, new GltfValidator_1.GltfValidator());
        ContentDataValidators.registerForBuffer(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_B3DM, new B3dmValidator_1.B3dmValidator());
        ContentDataValidators.registerForBuffer(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_I3DM, new I3dmValidator_1.I3dmValidator());
        ContentDataValidators.registerForBuffer(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_CMPT, new CmptValidator_1.CmptValidator());
        ContentDataValidators.registerForBuffer(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_PNTS, new PntsValidator_1.PntsValidator());
        // Certain content types are known to be encountered,
        // but are not (yet) validated. These can either be
        // ignored, or cause a warning. In the future, this
        // should be configurable, probably even on a per-type
        // basis, via the command line or a config file
        const ignoreUnhandledContentTypes = false;
        let geomValidator = Validators_1.Validators.createEmptyValidator();
        let vctrValidator = Validators_1.Validators.createEmptyValidator();
        let geojsonValidator = Validators_1.Validators.createEmptyValidator();
        if (!ignoreUnhandledContentTypes) {
            geomValidator = Validators_1.Validators.createContentValidationWarning("Skipping 'geom' validation");
            vctrValidator = Validators_1.Validators.createContentValidationWarning("Skipping 'vctr' validation");
            geojsonValidator = Validators_1.Validators.createContentValidationWarning("Skipping 'geojson' validation");
        }
        ContentDataValidators.register(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_GEOM, geomValidator);
        ContentDataValidators.register(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_VCTR, vctrValidator);
        ContentDataValidators.register(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_GEOJSON, geojsonValidator);
        ContentDataValidators.register(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_3TZ, ContentDataValidators.createArchiveValidator());
        ContentDataValidators.register(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_TILESET, ContentDataValidators.createTilesetValidator());
        ContentDataValidators.register(ContentDataTypes_1.ContentDataTypes.CONTENT_TYPE_GLTF, ContentDataValidators.createGltfJsonValidator());
        ContentDataValidators._registeredDefaults = true;
    }
    /**
     * Creates a validator for content data that refers to a 3TZ archive.
     *
     * This takes the contentData.uri, resolves it (to obtain an absolute URI),
     * and assumes that this URI is a path in the local file system, which
     * is then passed to the `TilesetArchiveValidator`
     *
     * @returns The validator
     */
    static createArchiveValidator() {
        const archiveValidator = new TilesetArchiveValidator_1.TilesetArchiveValidator();
        const validator = {
            async validateObject(inputPath, input, context) {
                const resourceResolver = context.getResourceResolver();
                const resolvedUri = resourceResolver.resolveUri(input.uri);
                const result = await archiveValidator.validateObject(inputPath, resolvedUri, context);
                return result;
            },
        };
        return validator;
    }
    /**
     * Creates a validator for content data that represents an external tileset.
     *
     * This validator will parse the JSON from the content data buffer, and
     * pass it to a default tileset validator.
     *
     * @returns The validator
     */
    static createTilesetValidator() {
        const externalValidator = Validators_1.Validators.createDefaultTilesetValidator();
        const bufferValidator = Validators_1.Validators.parseFromBuffer(externalValidator);
        const contentDataValidator = ContentDataValidators.wrapBufferValidator(bufferValidator);
        return contentDataValidator;
    }
    /**
     * Creates a validator for content data that represents glTF JSON data.
     *
     * This validator will pass the buffer with the JSON data to the
     * standard glTF validator.
     *
     * @returns The validator
     */
    static createGltfJsonValidator() {
        const bufferValidator = new GltfValidator_1.GltfValidator();
        const contentDataValidator = ContentDataValidators.wrapBufferValidator(bufferValidator);
        return contentDataValidator;
    }
    /**
     * Tries to find a data validator that can be used for validating
     * the given content data. If no matching validator can be found,
     * then `undefined` is returned.
     *
     * @param contentData - The `ContentData`
     * @returns The validator, or `undefined`
     */
    static async findContentDataValidator(contentData) {
        ContentDataValidators.registerDefaults();
        for (const entry of ContentDataValidators.dataValidators) {
            if (await entry.predicate(contentData)) {
                return entry.dataValidator;
            }
        }
        return undefined;
    }
    /**
     * Wraps the given validator for `Buffer` objects into one that
     * can be applied to `ContentData` objects, and just applies
     * the given validator to the buffer that is returned by
     * `ContentData#getData`.
     *
     * @param bufferValidator - The validator for `Buffer` objects
     * @returns The validator for `ContentData` objects
     */
    static wrapBufferValidator(bufferValidator) {
        return {
            async validateObject(inputPath, input, context) {
                const data = await input.getData();
                if (!(0, defined_1.defined)(data)) {
                    const message = `Could not resolve ${input.uri}`;
                    const issue = IoValidationIssue_1.IoValidationIssues.IO_WARNING(inputPath, message);
                    context.addIssue(issue);
                    return true;
                }
                const result = await bufferValidator.validateObject(inputPath, data, context);
                return result;
            },
        };
    }
    /**
     * Register a validator that should be used for the content
     * data buffer, when the content data matches the given
     * predicate.
     *
     * @param predicate - The predicate
     * @param bufferValidator - The validator for the buffer data
     */
    static registerForBuffer(predicate, bufferValidator) {
        ContentDataValidators.register(predicate, ContentDataValidators.wrapBufferValidator(bufferValidator));
    }
    /**
     * Registers a data validator that will be used when a
     * `ContentData` matches the given predicate.
     *
     * @param predicate - The predicate
     * @param dataValidator - The data validator
     */
    static register(predicate, dataValidator) {
        const entry = {
            predicate: predicate,
            dataValidator: dataValidator,
        };
        ContentDataValidators.dataValidators.push(entry);
    }
}
exports.ContentDataValidators = ContentDataValidators;
/**
 * The list of validators that have been registered.
 */
ContentDataValidators.dataValidators = [];
/**
 * Whether the default content data validators have already
 * been registered by calling 'registerDefaults'
 *
 * Note: This could be solved with a static initializer block, but the
 * unclear initialization order of the classes would make this brittle
 */
ContentDataValidators._registeredDefaults = false;
